/*
	Exemple de Laurent
	
	
	Utilisation de l'uART0 avec la librairie UART de Peter Fleury
	
*/

#define msTEMPO 1 // 1 simul 10 // Temporisation en ms

#include "mavin.h"
#include "ModuleFunction.h"

#include "uart.h"


/* define CPU frequency in Mhz here if not defined in Makefile */
#ifndef F_CPU
#define F_CPU 8000000UL
#endif


/* 9600 baud */
#define UART_BAUD_RATE      9600     

// Macro de temporisation
#define Tempo() mSecDelay(msTEMPO)


int main(void)
{
	// Local declaration
    unsigned int c;
	
	// Initialisation section
	
	Initialize(); // Initialize Mavin library
	
    /*
     *  Initialize UART library, pass baudrate and AVR cpu clock
     *  with the macro 
     *  UART_BAUD_SELECT() (normal speed mode )
     *  or 
     *  UART_BAUD_SELECT_DOUBLE_SPEED() ( double speed mode)
     */
    uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) ); 
    
    /*
     * now enable interrupt, since UART library is interrupt controlled
     */
    sei();


	
	
	// Main Loop
	
	while(1)
	{
		LcdDisplay( "Hello" , "World" );
		
		Tempo();
		




    /*
     *  Transmit string to UART
     *  The string is buffered by the uart library in a circular buffer
     *  and one character at a time is transmitted to the UART using interrupts.
     *  uart_puts() blocks if it can not write the whole string to the circular 
     *  buffer
     */
    uart_puts("String stored in SRAM\n");
    
    /*
     * Transmit string from program memory to UART
     */
    uart_puts_P("String stored in FLASH\n");

		// Repeat indefinitely

		for(;;)
		{
			/*
			 * Get received character from ringbuffer
			 * uart_getc() returns in the lower byte the received character and 
			 * in the higher byte (bitmask) the last receive error
			 * UART_NO_DATA is returned when no data is available.
			 *
			 */
			c = uart_getc();
			if ( c & UART_NO_DATA )
			{
				/* 
				 * no data available from UART 
				 */
			}
			else
			{
				/*
				 * new data available from UART
				 * check for Frame or Overrun error
				 */
				if ( c & UART_FRAME_ERROR )
				{
					/* Framing Error detected, i.e no stop bit detected */
					uart_puts_P("UART Frame Error: ");
				}
				if ( c & UART_OVERRUN_ERROR )
				{
					/* 
					 * Overrun, a character already present in the UART UDR register was 
					 * not read by the interrupt handler before the next character arrived,
					 * one or more received characters have been dropped
					 */
					uart_puts_P("UART Overrun Error: ");
				}
				if ( c & UART_BUFFER_OVERFLOW )
				{
					/* 
					 * We are not reading the receive buffer fast enough,
					 * one or more received character have been dropped 
					 */
					uart_puts_P("Buffer overflow error: ");
				}
				/* 
				 * send received character back
				 */
				uart_putc( (unsigned char)c );
			}
 		} // End for(;;)

		
	} // End while(1)

	
} // End main
